angular.module('beamng.apps')
	// register service
	.factory('VehicleService', function() {
		var vehicleName = "";
		return {
			getVehicleName: function() { 
				return vehicleName; 
			},
			setVehicleName: function(newVehicleName) 	{ 
				vehicleName = newVehicleName.replace(/_/gi," ")
				},
		};
	})
	.directive('distanceDisplay', ['bngApi', 'StreamsManager', 'UiUnits', 'VehicleService', function (bngApi, StreamsManager, UiUnits, VehicleService) {
		return {
			templateUrl: "modules/apps/separatedistancecounter/endurance.html",
			replace: true,
			restrict: 'EA',
			controller: ['$log', '$scope', 'bngApi', 'StreamsManager', function ($log, $scope, bngApi, StreamsManager) {
			  var streamsList = ['stats', 'sensors', 'electrics', 'engineInfo'];
			  StreamsManager.add(streamsList);
			  $scope.$on('$destroy', function () {
				StreamsManager.remove(streamsList);
			  });
			  
			  $scope.ignition = false;
			  
			  $scope.pitch = 0;
			  $scope.roll = 0;
			  $scope.yaw = 0;
			  
			  $scope.kph = 0;
			  $scope.mph = 0;
			  
			  $scope.weight = {
				total: '',
				wheels: { average: '', total: '', count: ''}
			  };
			  
			  $scope.beams = {
				total: '-',
				deformed: { number: '', percentage: ''},
				broken: { number: '', percentage: ''}
			  };
			  
			  $scope.torsionbars = {
				total: '-',
				deformed: { number: '', percentage: ''},
				broken: { number: '', percentage: ''}
			  };

			  $scope.$on('streamsUpdate', function (event, data) {
				$scope.beams.total    = data.stats.beam_count;
				$scope.beams.deformed = { number: data.stats.beams_deformed, percentage: data.stats.beams_deformed/data.stats.beam_count * 100 };
				$scope.beams.broken   = { number: data.stats.beams_broken, percentage: data.stats.beams_broken/data.stats.beam_count * 100 };
				
				$scope.weight.total  = data.stats.total_weight;
				$scope.weight.wheels = {
				  total: data.stats.wheel_weight,
				  count: data.stats.wheel_count,
				  average: data.stats.wheel_weight / data.stats.wheel_count
				};
				$scope.weight.chassis = data.stats.total_weight - data.stats.wheel_weight;
				
				$scope.torsionbars.total = data.stats.torsionbar_count;
				$scope.torsionbars.deformed = { number: data.stats.torsionbars_deformed, percentage: data.stats.torsionbars_deformed/data.stats.torsionbar_count * 100 };
				$scope.torsionbars.broken = { number: data.stats.torsionbars_broken, percentage: data.stats.torsionbars_broken/data.stats.torsionbar_count * 100 };
				
				$scope.ignitionBool = data.electrics.ignition;
				$scope.runningBinary = data.electrics.engineRunning;
				
				$scope.oilTemp = data.electrics.oiltemp;
				$scope.waterTemp = data.electrics.watertemp;
				$scope.oilTemp2 = (data.electrics.oiltemp * 9 / 5) + 32;
				$scope.waterTemp2 = (data.electrics.watertemp * 9 / 5) + 32;
				
				$scope.currentRPM = data.electrics.rpm;
				$scope.idleRPM = data.engineInfo[0];
				$scope.maxRPM = data.engineInfo[1];
				$scope.meterRPM = (100 / data.engineInfo[1]) * data.electrics.rpm;
				document.getElementById('revMeter').style.width = Math.round($scope.meterRPM) + "%";
				
				var rgb = '';

                if($scope.currentRPM < data.engineInfo[0] * 1.25) { //we are at idle, blue
                    rgb = '(158, 214, 241';
                } else if($scope.currentRPM > data.engineInfo[1] * 0.9) { //we are near redline, red
                    rgb = '(241, 158, 158';
                } else { //normal rpm, green
                    rgb = '(186, 241, 158';
                }
				
				document.getElementById('revMeter').style.background = 'rgba' + rgb + ',0.5)';
				
				if (data.electrics.gear == 0) {
				  $scope.currentGear = 'N';
				} else if (data.electrics.gear < 0) {
				  $scope.currentGear = 'R';
				} else {
				  $scope.currentGear = data.electrics.gear;
				}
				
				$scope.pitch = Math.asin(data.sensors.pitch) * (180 / Math.PI);
				$scope.roll = Math.asin(data.sensors.roll) * (180 / Math.PI);
				$scope.yaw = (data.sensors.yaw * 180 / Math.PI + 180);
				
				$scope.kph = Math.round((data.electrics.wheelspeed * 3.6) * 10) / 10;
				$scope.mph = Math.round(($scope.kph / 1.609344) * 10) / 10;
				
			  });
			}],
			link: function (scope, element, attrs) {
				'use strict';
							
				var state;
				
				var formatter = new Intl.NumberFormat('en-US', {
				  style: 'currency',
				  currency: 'USD',
				});
				
				
				
				scope.progress = 0;
				scope.progress2 = 0;
				scope.progressEst = 0;
				scope.progress2Est = 0;
				scope.fuelPercent = 0;
				scope.fuelCap = 0;
				scope.fuelVol = 0;
				scope.fuelLperKM = 0;
				scope.fuelMperG = 0;
				
				scope.tSpeed = 0;
				scope.tSpeed2 = 0;
				
				scope.endurancePoints = 0;

				// assign vehicleName to currently stored value in VehicleService
				scope.vehicleName = VehicleService.getVehicleName();
				
				scope.$on("VehicleReset", function(event, data) {
					// get current vehicle
					bngApi.engineLua("core_vehicles.getCurrentVehicleDetails()", function(vehicle) {
						scope.$evalAsync(function() {
							// save current vehicle key
							VehicleService.setVehicleName(vehicle.current.key);
							scope.vehicleName = VehicleService.getVehicleName();
						})
					})
				});
				
				scope.$on("VehicleChange", function(event, data) {
					// get current vehicle
					bngApi.engineLua("core_vehicles.getCurrentVehicleDetails()", function(vehicle) {
						scope.$evalAsync(function() {
							// save current vehicle key
							VehicleService.setVehicleName(vehicle.current.key);
							scope.vehicleName = VehicleService.getVehicleName();
						})
					})
				});

				scope.$on('VehicleFocusChanged', function (event, data) {
					// get current vehicle
					bngApi.engineLua("core_vehicles.getCurrentVehicleDetails()", function(vehicle) {
						scope.$evalAsync(function() {
							// save current vehicle key
							VehicleService.setVehicleName(vehicle.current.key);
							scope.vehicleName = VehicleService.getVehicleName();
						})
					})
				});
				
				scope.$on('sendState', function (event, data) {
					state = data;
					
					scope.vehicleModel = data.carModel;
					scope.vehicleConfig = data.carConfig;
					
					scope.tSpeed = Math.round(data.tSpeed * 10) / 10;
					scope.tSpeed2 = Math.round((data.tSpeed / 1.609344) * 10) / 10;
				  
					scope.progress = Math.round((data.progress / 1000) * 100) / 100;
					scope.progress2 = Math.round((data.progress / 1609.3440057765) * 100) / 100;
					scope.progressEst = Math.round(((scope.fuelVol / scope.fuelLperKM) * 100) * 10) / 10;
					scope.progressEst2 = Math.round((scope.progressEst / 1.6093440057765) * 10) / 10;
					
					scope.fuelPercent = Math.round((data.fuelPercent * 100) * 10) / 10;
					scope.fuelCap = data.fuelCap;
					scope.fuelVol = Math.round(data.fuelVol * 100) / 100;
					scope.fuelLperKM = Math.round((((scope.fuelCap - scope.fuelVol) / scope.progress) * 100) * 10) / 10;
					scope.fuelMperG = Math.round((235.214583 / scope.fuelLperKM) * 10) / 10;
					
					scope.endurancePoints = formatter.format(data.progress / scope.fuelLperKM * (100 - scope.beams.deformed.percentage));
					
					scope.$apply();
				});	

				scope.$on('streamsUpdate', function (event, streams) {
					scope.$evalAsync(function () {
						bngApi.activeObjectLua('extensions.separatedistancecounter.getVar()');
						
					});
				});
								
			}
		};
}])
